#!/bin/sh
#
#  Copyright IBM Corp. 2015-2017
#  All rights reserved.
#  US Government Users Restricted Rights -
#  Use, duplication or disclosure restricted
#  by GSA ADP Schedule Contract with IBM Corp.
#
#  Licensed Materials-Property of IBM
#
echo "Running JDK upgrade script --- installServiceforCB.sh"
dos2unix *.properties
####################################################################
# Get saved values. Anything in service.properties will override
# Source the WHV virtualimage.properties file
####################################################################
. /etc/virtualimage.properties

if  [ "" != "$WAS_BITNESS" ]; then
    BITNESS=$WAS_BITNESS
    echo "Found bitness in source env=$WAS_BITNESS"
elif [ -r $PROFILES_HOME/.ibm/bitness ]; then
    BITNESS=`cat $PROFILES_HOME/.ibm/bitness`
    echo "Found bitness file:$PROFILES_HOME/.ibm/bitness  set bitness to: $BITNESS"
else
    echo "bitness file: $PROFILES_HOME/.ibm/bitness not found trying system bitness check"
    $WAS_HOME/bin/versionInfo.sh |grep "32 bit"  >/dev/null
    bit32result=$?
    $WAS_HOME/bin/versionInfo.sh |grep "64 bit" >/dev/null
    bit64result=$?
    if [ $bit64result == 0 ] ; then 
        BITNESS="64"
    elif [ $bit32result == 0 ] ; then 
        BITNESS="32"
    else 
       echo "Dumping bitness command = `$WAS_HOME/bin/versionInfo.sh |grep "bit"`  "
       echo "ERROR: Unable to determine system bitness"
       exit 1
    fi 
    echo "bitness returned $BITNESS"

fi

####################################################################
# Determine arch Linux | AIX
###################################################################
if [ -f /etc/SuSE-release ] || [ -f /etc/redhat-release ] ; then
    ARCH="LinuxX$BITNESS"
    echo "The target system is LinuxX$BITNESS"
elif [ -f /etc/motd ]; then
    grep -q AIX /etc/motd
    result=$?
    if [ $result == "0" ] ; then
    	ARCH="AixPPC$BITNESS"
        echo "The target system is AixPPC$BITNESS"
    fi
else
    echo "ERROR: The target operating system is not supported by this interim fix"
    exit 1
fi
####################################################################
# source the properties, with bitness and arch replaced with values above 
###################################################################

if [ -f ./service.properties ]; then
    . ./service.properties
else
    echo "ERROR: Cannot read service.properties."
    exit 1
fi
####################################################################
# Determine proper Userid for script
###################################################################

#Default the install user if not saved 
if  [ "" == "$INSTALL_USER" ]; then
    INSTALL_USER="virtuser"
fi

# WHV_USER is the new value, but we can always find it in username
if [ -z "$WHV_USER" ]; then
    if [ -r $PROFILES_HOME/.ibm/username ]; then
        WHV_USER=`cat $PROFILES_HOME/.ibm/username`
    else
        WHV_USER="virtuser"
    fi
fi

####################################################################
# Setup IM installer Repository URL
####################################################################

# Default the next two if not already specified
if [ -z "$IM_HOME" ]; then
    IM_HOME="/home/$INSTALL_USER/IBM/InstallationManager/eclipse/"
fi
if [ -z "$REPOSITORY_LOCATION" ]; then
    REPOSITORY_LOCATION="http://IBMWorkloadDeployer:8585/IMRepository/Compo"
fi

####################################################################
# Setup IM installer Credential variables
####################################################################
SERVICE_DIR=`pwd`
chmod 777 $SERVICE_DIR
if [ -f $IM_HOME/tools/master_password_file.txt ]; then
    echo "InstallationManager is configured with credentials"
    CREDENTIALS="-secureStorageFile $IM_HOME/tools/credential.store -masterPasswordFile $IM_HOME/tools/master_password_file.txt"
else
    echo "InstallationManager is NOT configured with credentials"
    CREDENTIALS=""
fi

#####################################################################
# List current installed packages 
#####################################################################
get_installed_packages()
{
   echo "======================================="
   su $INSTALL_USER -c "$IM_HOME/tools/imcl listInstalledPackages >$SERVICE_DIR/installed.log"
   su $INSTALL_USER -c "$IM_HOME/tools/imcl listInstalledPackages -long >$SERVICE_DIR/installedlong.log"
   STEP_RESULT=$?
   if [ $? -ne 0 ]; then
       echo "ERROR: InstallationManager to listInstalledPackages: return code $?"
       echo "STEP_RESULT: $STEP_RESULT"
       exit 1
   fi
   echo "Currently installed packages:"
   cat "$SERVICE_DIR/installed.log"
   echo "======================================="
}

####################################################################
# List Available fix packs that are currently are available in the repo
#
#  Parms:
#     $1=package offering id
#     $2=package offering service version 
#     $3=Generic package query to force imcl to find ALL version fixes 
#  Script will exit rc=1 if the IFIX service version package
#  is not available in the repository

####################################################################
find_ifix_repo()
{
   IPACKAGE=$1
   IVERSION=$2
   IFIXQUERY=$3  
   echo "======================================="
   echo "Running imcl IFIX repo search"
   
   echo "su $INSTALL_USER -c $IM_HOME/tools/imcl listAvailableFixes ${IPACKAGE}_${IFIXQUERY} $CREDENTIALS -repositories $REPOSITORY_LOCATION >$SERVICE_DIR/iFIXrepo.log"
   su $INSTALL_USER -c      "$IM_HOME/tools/imcl listAvailableFixes ${IPACKAGE}_${IFIXQUERY} $CREDENTIALS -repositories $REPOSITORY_LOCATION >$SERVICE_DIR/iFIXrepo.log"
   STEP_RESULT=$?
   if [ $? -ne 0 ]; then
       echo "ERROR: InstallationManager listAvailableFixes Query failed return code $?"
       echo "STEP_RESULT: $STEP_RESULT"
       exit 1
   fi
   echo "The installation repository contains the following packages:"
   cat  $SERVICE_DIR/iFIXrepo.log
   cat  $SERVICE_DIR/iFIXrepo.log >>$SERVICE_DIR/appendRepo.log
   echo "======================================="
}

#####################################################################
# java running checklifecycle 
#
#  No parms: Check if all WAS processes have stopped 
#####################################################################
java_running_delay()
{
   echo "Now checking for any running processes"
   cnt=`ps ax | grep $WAS_HOME | grep -v -c "grep"`
   if [ "$cnt" -gt 0 ]
   then
       ps ax | grep $WAS_HOME | grep -v "grep"
       echo "Found $cnt processes. Sleeping 120 seconds to allow them to complete ..."
       sleep 120
       echo "Killing any blocking processes"
       ps ax | grep $WAS_HOME | grep -v "grep"
       ps ax | grep $WAS_HOME | grep -v "grep" | awk '{print $1}' | xargs kill -9 >/dev/null 2>&1
   else
       echo "No processes found"
   fi
}

#####################################################################
# IHS lifecycle 
#
# $1= Stop / Start 
#####################################################################
ihs_lifecycle()
{
   ISERVICE=$1
   # The IHS variable is set the first time the start lifecycle runs
   if [ -z $IHS ]; then
      return
   fi

   #  IHS. Ignore errors, because server may already be stopped
   echo "======================================="
   if [ -f "$IHS_HOME/bin/apachectl" ]; then
      echo "$ISERVICE IHS"
      if [ "$ISERVICE" == "STOP" ]; then
         $IHS_HOME/bin/apachectl stop
         $IHS_HOME/bin/adminctl stop
         # Give 30 seconds for java to process to settle.
         java_running_delay
      else  
         $IHS_HOME/bin/apachectl start
         $IHS_HOME/bin/adminctl start
      fi 
   fi
}

#####################################################################
# Liberty lifecycle 
#
# $1= Stop / Start 
#####################################################################
Liberty_lifecycle()
{
   ISERVICE=$1

   #Liberty. Ignore errors, because server may already be stopped or server not created
   if [ -d "$PROFILES_HOME/Liberty/servers" ]; then
   echo "======================================="
      echo "$ISERVICE Liberty"
      cd "$PROFILES_HOME/Liberty/servers"
      for server in `ls -d *`; do
         echo "$ISERVICE Liberty server $server"
         echo "If configuration has not completed an error will be displayed and ignored"
         if [ "$ISERVICE" == "STOP" ]; then
            su $WHV_USER -c "$WLP_HOME/bin/server stop $server"
         else 
            su $WHV_USER -c "$WLP_HOME/bin/server start $server"
         fi
      done
      cd "$SERVICE_DIR"
   fi
}

#####################################################################
# WebSphere lifecycle 
#
# $1= Stop / Start 
#####################################################################

websphere_lifecycle()
{
   ISERVICE=$1
   echo "======================================="
   echo "$ISERVICE WebSphere"

   for profile in `ls -d $PROFILES_HOME/*/`; do
       if [ -f "$profile/bin/stopServer.sh" ]; then
           echo "$ISERVICE servers in profile $profile"
           if [ -d $profile/servers ]; then
             cd "$profile/servers"
             for server in `ls -d *`; do
                 echo $server | grep "webserver" >/dev/null
                 if [ $? -ne 0 ]; then
                    echo $server | grep "ffdc" >/dev/null
                    if [ $? -ne 0 ]; then
                       echo "If configuration has not completed an error will be displayed and ignored"
                       if [ "$PROCESS_RUNAS_ROOT" == "TRUE" ]; then
                          if [ "$ISERVICE" == "STOP" ]; then
                             $profile/bin/stopServer.sh $server
                             # Give some time for java to process to exit
                             java_running_delay
                          else 
                  	     $profile/bin/startServer.sh $server 
                          fi
                       else 
                          if [ "$ISERVICE" == "STOP" ]; then
                             su $WHV_USER -c "$profile/bin/stopServer.sh $server"
                             # Give some time for java to process to exit
                             java_running_delay
                          else
                             su $WHV_USER -c "$profile/bin/startServer.sh $server"
                          fi
                       fi
                    fi
                 fi
             done
         else
            echo "No servers defined on this node."
         fi
       fi
   done
}

####################################################################
# Locate a package in the list of available
#  Parms:
#     $1=package offering id
#     $2=package offering service version 
#  Script will exit rc=1 if the service version 
#  is not available in the repository
####################################################################
findpackage()
{
  IPACKAGE=$1
  IVERSION=$2
  IFIXQUERY=$3

  find_ifix_repo "$IPACKAGE" "$IVERSION" "$IFIXQUERY"

  cat $SERVICE_DIR/iFIXrepo.log | grep $IVERSION >/dev/null
  if [ $? -ne 0 ]; then
      echo "ERROR: Repository does not contain an available iFix Version $IVERSION" | sed 's/\\//g'
      echo "ERROR: The installed package cannot be upgraded $IPACKAGE" | sed 's/\\//g'
      exit 1
  fi
}

####################################################################
# Locate a package in the list of installed
####################################################################
findinstalled()
{
  IPACKAGE=$1

  cat $SERVICE_DIR/installed.log | grep $IPACKAGE >/dev/null
  if [ $? -ne 0 ]; then
      echo "Package $IPACKAGE is NOT currently installed" | sed 's/\\//g'
      return 0
  fi
  return 1
}

####################################################################
#  Check if we need to service this component
#  Parms:
#     $1=package offering id
#     $2=package offering service version (not the currently installed version)
#  Return:
#     0=Do not service this component
#     1=Service this component, and version is available
#  Script will return rc=1 if the package is installed, but the service version 
#  is not available in the repository
####################################################################
servicepackage()
{
    IPACKAGE=$1
    IVERSION=$2
    IFIXQUERY=$3

    # If version not specified, then we will not service this component
    if [ -z "$IVERSION" ]; then
        echo "Version not specified: $IVERSION"
        return 0
    fi

    # Is this package installed on this VM? 
    findinstalled "$IPACKAGE"
    if [ $? -ne 0 ]; then
        echo "Package is installed: $IPACKAGE"
        if [ -z "$IFIXQUERY" ]; then
            echo "This is not a fix, it is a fixpack"
        else
            findpackage "$IPACKAGE" "$IVERSION" "$IFIXQUERY"
        fi

        # If yes, then return 1
        return 1
    else
        # If no just return 0
        return 0
    fi
}

####################################################################
#  Update the java.security file
#  Parms:
#     $1=IM installation dir
#     $2=subdirectory under that for this java
#  Return:
#     0=java directory not specified
####################################################################
updatejava()
{
    IBASE=$1
    IJAVA=$2


    # If IJAVA not specified, then we will not update
    if [ -z "$IJAVA" ]; then
        echo "Updating security file not required for this package."
        return 0
    fi
   
    SFILE="$IBASE/$IJAVA/jre/lib/security/java.security"
    echo "start security update"
    echo "# Inserted by JDK Update eFix"  >>$SFILE
    echo "jdk.tls.disabledAlgorithms=SSLv3, RC4, DH keySize <768, MD5withRSA" >>$SFILE 
    echo "jdk.certpath.disabledAlgorithms=MD2, RSA keySize <1024, MD5" >>$SFILE
    
}

####################################################################
# Uninstall JDK  
#  Parms:
#     $1=package offering id
#     $2=offering service version 
####################################################################
uninstall_jdk(){
   echo "======================================="
   cat $SERVICE_DIR/installedlong.log | grep $ARCH >$SERVICE_DIR/installedjava.log
   if [ $? -ne 0 ]; then
      echo "No JDK iFix was found to uninstall"
   else 
      echo "Uninstalling previous JDK iFixes"
      while IFS='' read -r installedPackage || [[ -n "$installedPackage" ]]; do
         # Get the fix and installed location
         ifix=`echo $installedPackage | awk -F ": " '{print $2}'`
         ilocation=`echo $installedPackage | sed "s/ :.*//"`
         echo "======================================="
         echo "Uninstalling JDK iFix $ifix from $ilocation"
         echo "su $INSTALL_USER -c $IM_HOME/tools/imcl uninstall $ifix -installationDirectory $ilocation  -log $SERVICE_DIR/IMuninstall.log"
         su $INSTALL_USER -c "$IM_HOME/tools/imcl uninstall $ifix -installationDirectory $ilocation -log $SERVICE_DIR/IMunstall.log"
         STEP_RESULT=$?
         if [ $? -ne 0 ]; then
            echo "ERROR: Uninstall of JDK iFix failed"
            echo "STEP_RESULT: $STEP_RESULT"
            exit $STEP_RESULT
         fi
         let uninstalled+=1
         cat "$SERVICE_DIR/IMunstall.log"
         echo "STEP_RESULT: $STEP_RESULT"
      done < $SERVICE_DIR/installedjava.log

      if [ $uninstalled -ne 0 ]; then
         echo "Uninstalled $uninstalled iFixes"
         echo "Verify installed packages after uninstall:"
         get_installed_packages
      fi

      echo "Completed JDK iFix uninstall"

      return $STEP_RESULT

   fi

}

####################################################################
# upgrade JDK  
#  Parms:
#     $1=package offering id
#     $2=offering service version 
#     $3= location ifix install directory
#     $4= Java Security some JDKs require manual file updates.  
####################################################################
upgrade_jdk()
{

   IPACKAGE=$1
   IVERSION=$2
   IFIXLOCATION=$3
   IFIXSECURITY=$4
   echo "======================================="
   echo " Upgrading JDK with $IVERSION"
 
   echo "su $INSTALL_USER -c $IM_HOME/tools/imcl install $IVERSION -repositories $REPOSITORY_LOCATION -acceptLicense -installationDirectory $IFIXLOCATION -log $SERVICE_DIR/IMupdate.log $CREDENTIALS -preferences com.ibm.cic.common.core.preferences.preserveDownloadedArtifacts=false"
   su $INSTALL_USER -c      "$IM_HOME/tools/imcl install $IVERSION -repositories $REPOSITORY_LOCATION -acceptLicense -installationDirectory $IFIXLOCATION -log $SERVICE_DIR/IMupdate.log $CREDENTIALS -preferences com.ibm.cic.common.core.preferences.preserveDownloadedArtifacts=false"
   STEP_RESULT=$?
   cat "$SERVICE_DIR/IMupdate.log"
   echo "STEP_RESULT: $STEP_RESULT"
   if [ $STEP_RESULT -eq 0 ]; then
      let installed+=1
      echo "Install $IVERSION SUCCESS"
      echo "======================================="
      echo "Check if Java security needs Upgrading"      
      updatejava "$IFIXLOCATION" "$IFIXSECURITY"  
      return $STEP_RESULT
   else 
      # This is the message you get if JDK 6 is installed and we are trying to install JDK 8 fix
      cat "$SERVICE_DIR/IMupdate.log" | grep "is not applicable" >/dev/null
      if [ $? -eq 0 ]; then
         let not_applicable+=1
         echo "Fix $IVERSION was not applicable"
         return 0
      fi
      # This is the message you get if JDK 8 is installed and we are trying to install JDK 6 fix
      cat "$SERVICE_DIR/IMupdate.log" | grep "does not apply to the current installation." >/dev/null
      if [ $? -eq 0 ]; then
         let not_applicable+=1
         echo "Fix $IVERSION was not applicable"
         return 0
      else
         echo "ERROR: Install $IVERSION Failed"
         echo "======================================="
      fi
      return $STEP_RESULT
   fi
}

####################################################################
# Main Install Code loop - cycle through additional JDK installs
####################################################################

result=0
anyfail=0
installed=0
uninstalled=0
not_applicable=0
SERVICE_DIR=`pwd`
PACKAGE_VALUE=""

# stop everything just once
websphere_lifecycle "STOP"
Liberty_lifecycle "STOP"
ihs_lifecycle "STOP"
cd $SERVICE_DIR

get_installed_packages
uninstall_jdk

# Read product file line by line into var $product 
# Example line: JDK70_PACKAGE JDK70_VERSION JDK70_LOCATION JDK70_QUERY JDK70_SECURITY

echo "Starting Main Upgrade loop"

while IFS='' read -r product || [[ -n "$product" ]]; do
   # Convert string $product to a sourced var example $JDK70_PACKAGE
   # eval pulls the actual $JDK70_PACKAGE into a local loop var.
   # awk parses the loop string per column see example string above.
   PACKAGE_VALUE=$(echo $product |  awk '{print $1}')
   eval DISP_PACK=\$${PACKAGE_VALUE}
   VERSION_VALUE=$(echo $product |  awk '{print $2}')
   eval DISP_VER=\$${VERSION_VALUE}
   LOCATION_VALUE=$(echo $product |  awk '{print $3}')
   eval DISP_LOC=\$${LOCATION_VALUE}
   PACKAGE_QUERY=$(echo $product |  awk '{print $4}')
   eval DISP_QUERY=\$${PACKAGE_QUERY}
   JAVA_SECURITY=$(echo $product |  awk '{print $5}')
   eval DISP_SECURITY=\$${JAVA_SECURITY}

   servicepackage "$DISP_PACK" "$DISP_VER" "$DISP_QUERY"
   if [ $? -ne 0 ]; then
      #Packages and Repo had required packages so proceed with upgrade process. 
      cat $SERVICE_DIR/installedlong.log | grep $DISP_PACK | sed "s/ :.*//" > $SERVICE_DIR/location.log

      while IFS='' read -r LOCATION_VALUE || [[ -n "$LOCATION_VALUE" ]]; do
        upgrade_jdk "$DISP_PACK" "$DISP_VER" "$LOCATION_VALUE" "$DISP_SECURITY"
        stepFail=$?
        if [ $stepFail -ne 0 ]; then
          echo "ERROR: Upgrade JDK with $DISP_PACK $DISP_VER failed!!"
          let anyfail+=1
        fi
      done < $SERVICE_DIR/location.log

   else
      echo "$DISP_PACK is not installed"
   fi
done < $SERVICE_DIR/productList.properties

# Start everything again
websphere_lifecycle "START"
Liberty_lifecycle "START"
ihs_lifecycle "START"
cd $SERVICE_DIR

echo "======================================="
echo "Installed packages at end of script:"
su $INSTALL_USER -c "$IM_HOME/tools/imcl listInstalledPackages"

#Determine pass / fail
echo "***************************************"
#Check if there have been any fails
echo "$installed iFixes were installed."
echo "$not_applicable iFixes were not applicable"
echo "$uninstalled iFixes were uninstalled"
if [ $anyfail -eq 0 ]; then
   echo "INSTCONFSUCCESS"
   result=0
else
   if [ $installed -eq 0 ]; then
       echo "There were $anyfail failures during installation."
       echo "Check logs for failure details"
       echo "INSTCONFFAIL"
       result=$anyfail 
   else
       echo "iFixes were installed, but there were $anyfail failures."
       echo "Check logs for failures"
       echo "If a failure occurred on an iFix that has a supersede iFix already installed"
       echo "or a JDK6/JDK8 mismatch, those should not be considered an error."
       echo "INSTCONFPARTIAL"
       result=$anyfail 
   fi
fi 
echo "***************************************"

echo "Clean up" 
cd /tmp/
rm -rf /tmp/update
exit $result

